import com.j256.ormlite.dao.Dao;

import java.sql.SQLException;
import java.util.Scanner;

public class AccountService {

    boolean quit = false;
    Scanner scanner = new Scanner(System.in);
    String input;
    Dao<Account, Integer> accountDao;

    public AccountService(Dao<Account, Integer> accountDao) {
        this.accountDao = accountDao;
    }

    /**
     * Read and write some example data.
     */
    void processData() throws Exception {
        while (quit == false) {
            commandSelect("Please choose an action:\n", "'c' create; 'r' read; 'u' updateSelect; 'd' delete; 'q' quit");
            switch (input) {
                case "c": setupAccount(); break;
                case "r": accountToString(); break;
                case "u": updateSelect(); break;
                case "d": delete(); break;
                case "q": quit(); break;
                default: System.out.println("Invalid input.\n" + "'c' create; 'r' read; 'u' updateSelect; 'd' delete; 'q' quit");
            }
        }
    }

    int setupAccount() throws SQLException {
        System.out.println("Please name the account.");
        input = scanner.nextLine();
        Account account = new Account(input);
        System.out.println("Please create a password.");
        input = scanner.nextLine();
        account.setPassword(input);
        int result = create(account);
        int id = account.getId();
        System.out.println(String.format("Account created with id %d", id));
        return  result;
    }

    int create(Account account) throws SQLException {
        return accountDao.create(account);
    }

    String accountToString() throws SQLException {
        Account account = fetchAccount();
        String accountInfo;
        try {accountInfo = String.format("ID: %d; Name: %s; Password: %s", account.getId(), account.getName(), account.getPassword());}
        catch (NullPointerException npe) {
            System.out.println("Sorry! No such account exists.");
            return null;
        }
        System.out.println(accountInfo);
        return accountInfo;
    }

    private Account fetchAccount() throws SQLException {
        System.out.println("Please provide an account ID.");
        input = scanner.nextLine();
        return read(Integer.parseInt(input));
    }

    Account read(int id) throws SQLException {
        return accountDao.queryForId(id);
    }

    void updateSelect() throws SQLException {
        Account account = fetchAccount();
        commandSelect("What would you like to updateSelect?\n", "'n' name, 'p' password");
        switch (input) {
            case "n": updateName(account); break;
            case "p": updatePassword(account); break;
            default: System.out.println("Invalid input.\n" +"'n' name, 'p' password");
        }
        System.out.println(String.format("Account %d updated.", account.getId()));
    }

    private void commandSelect(String s, String s2) {
        System.out.println(s
                + s2);
        input = scanner.nextLine();
    }

    private void updatePassword(Account account) throws SQLException {
        System.out.println("Please enter a new password.");
        account.setPassword(scanner.nextLine());
        update(account);
    }

    int update(Account account) throws SQLException {
        return accountDao.update(account);
    }

    private void updateName(Account account) throws SQLException {
        System.out.println("Please enter a new name.");
        account.setName(scanner.nextLine());
        update(account);
    }

    int delete() throws SQLException {
        Account account = fetchAccount();
        System.out.println(String.format("Deleting account %d.", account.getId()));
        return delete(account);
    }

    int delete(Account account) throws SQLException {
        return accountDao.delete(account);
    }

    void quit() {
        quit = true;
    }

}
